/*
     PLIB - A Suite of Portable Game Libraries
     Copyright (C) 2001  Steve Baker
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Library General Public License for more details.
 
     You should have received a copy of the GNU Library General Public
     License along with this library; if not, write to the Free
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
     For further information visit http://plib.sourceforge.net

     $Id: fntTXF.cxx,v 1.9 2001/04/09 00:24:58 deca Exp $
*/

// Modified for Dreamcast/KOS by Peter Hatch

#include "fntLocal.h"

FILE *curr_image_fd;
int isSwapped = false;

bool fntTexFont::loadTXF (const char *fname)
{
  FILE *fd;

  if ( (fd = fopen (fname, "rb") ) == NULL)
	{
	  ulSetError (UL_WARNING, "fntLoadTXF: Failed to open '%s' for reading.", fname);
	  return false;
	}

  curr_image_fd = fd;

  unsigned char magic[4];

  if ( (int)fread (&magic, sizeof (unsigned int), 1, fd ) != 1)
	{
	  ulSetError (UL_WARNING, "fntLoadTXF: '%s' an empty file!", fname);
	  return false;
	}

  if (magic[0] != 0xFF || magic[1] != 't' || magic[2] != 'x' || magic[3] != 'f')
	{
	  ulSetError ( UL_WARNING, "fntLoadTXF: '%s' is not a 'txf' font file.", fname);
	  return false;
	}

  isSwapped = false;
  int endianness = _fnt_readInt ();

  isSwapped = (endianness != 0x12345678);

  int format = _fnt_readInt ();
  tex_width = _fnt_readInt ();
  tex_height = _fnt_readInt ();
  int max_height = _fnt_readInt ();
  _fnt_readInt ();
  int num_glyphs = _fnt_readInt ();

  int w = tex_width;
  int h = tex_height;

  float xstep = 0.5f / (float) w;
  float ystep = 0.5f / (float) h;

  /*
    Load the TXF_Glyph array
  */

  TXF_Glyph glyph;

  for (int i = 0; i < num_glyphs; i++)
	{
	  glyph.ch = _fnt_readShort ();

	  glyph.w = _fnt_readByte ();
	  glyph.h = _fnt_readByte ();
	  glyph.x_off = _fnt_readByte ();
	  glyph.y_off = _fnt_readByte ();
	  glyph.step = _fnt_readByte ();
	  glyph.unknown = _fnt_readByte ();
	  glyph.x = _fnt_readShort ();
	  glyph.y = _fnt_readShort ();

	  setGlyph ( (char)   glyph.ch,
				 (float) glyph.x                / (float) w + xstep,
				 (float)(glyph.x + glyph.w)     / (float) w + xstep,
				 (float) glyph.y                / (float) h + ystep,
				 (float)(glyph.y + glyph.h)     / (float) h + ystep,
				 (float) glyph.x_off            / (float) max_height,
				 (float)(glyph.x_off + glyph.w) / (float) max_height,
				 (float) glyph.y_off            / (float) max_height,
				 (float)(glyph.y_off + glyph.h) / (float) max_height);
	}

  exists[' '] = false;

  /*
    Load the image part of the file
  */

  int ntexels = w * h;

  uint8 *teximage;
  unsigned char *texbitmap;

  switch (format)
	{
    case FNT_BYTE_FORMAT:
	{
	  unsigned char *orig = new unsigned char [ntexels];

	  if ( (int)fread (orig, 1, ntexels, fd) != ntexels)
        {
          ulSetError ( UL_WARNING, "fntLoadTXF: Premature EOF in '%s'.", fname);
          return false;
        }

	  teximage = new uint8 [ntexels * 2];

	  for (int i = 0; i < ntexels; i++)
        {
		  uint8 argb = orig [i] >> 4;
		  argb = argb + (argb << 4);
          teximage [i * 2] = argb;
          teximage [i * 2 + 1] = argb;
        }

	  delete orig;

	}
	break;
   
    case FNT_BITMAP_FORMAT:
	{
	  int stride = (w + 7) >> 3;

	  texbitmap = new unsigned char[stride * h];

	  if ( (int)fread (texbitmap, 1, stride * h, fd) != stride * h)
        {
          delete texbitmap;
          ulSetError (UL_WARNING, "fntLoadTXF: Premature EOF in '%s'.", fname);
          return false;
      	}

	  teximage = new uint8[ntexels * 2];

	  memset (teximage, 0, ntexels * 2);

	  for (int i = 0; i < h; i++)
		{
		  for (int j = 0; j < w; j++)
			{
			  if (texbitmap[i * stride + (j >> 3)] & (1 << (j & 7) ) )
				{
				  teximage[(i * w + j) * 2] = 0xFF;
				  teximage[(i * w + j) * 2 + 1] = 0xFF;
				}
			}
		}
	  
	  delete texbitmap ;
	}
	break;

    default:
      ulSetError (UL_WARNING, "fntLoadTXF: Unrecognised format type in '%s'.", fname);
      return false;
	}

  fclose (fd);

  fixed_pitch = false;

  texture = ta_txr_allocate (ntexels * 2);
  ta_txr_load (texture, teximage, ntexels * 2);

  return true;
}
